@@ -13,7 +13,9 @@ from __future__ import absolute_import
 # Stdlib imports
 import os
 import os.path as osp
+import re
 from string import Template
+import sys
 import time
 
 # Qt imports
@@ -22,18 +24,22 @@ from spyderlib.qt.QtGui import (QTextEdit, QKeySequence, QWidget, QMenu,
                                 QMessageBox)
 from spyderlib.qt.QtCore import SIGNAL, Qt
 
+from spyderlib import pygments_patch
+pygments_patch.apply()
+
 # IPython imports
-try:  # 1.0
-    from IPython.qt.console.rich_ipython_widget import RichIPythonWidget
-except ImportError: # 0.13
-    from IPython.frontend.qt.console.rich_ipython_widget import RichIPythonWidget
+from IPython.qt.console.rich_ipython_widget import RichIPythonWidget
+from IPython.qt.console.ansi_code_processor import ANSI_OR_SPECIAL_PATTERN
+from IPython.core.application import get_ipython_dir
 from IPython.core.oinspect import call_tip
+from IPython.config.loader import Config, load_pyconfig_files
 
 # Local imports
 from spyderlib.baseconfig import (get_conf_path, get_image_path,
                                   get_module_source_path, _)
 from spyderlib.config import CONF
-from spyderlib.guiconfig import create_shortcut, get_font, get_shortcut
+from spyderlib.guiconfig import (create_shortcut, get_font, get_shortcut,
+                                 new_shortcut)
 from spyderlib.utils.dochelpers import getargspecfromtext, getsignaturefromtext
 from spyderlib.utils.qthelpers import (get_std_icon, create_toolbutton,
                                        add_actions, create_action, get_icon,
@@ -43,6 +49,8 @@ from spyderlib.widgets.browser import WebView
 from spyderlib.widgets.calltip import CallTipWidget
 from spyderlib.widgets.mixins import (BaseEditMixin, InspectObjectMixin,
                                       SaveHistoryMixin, TracebackLinksMixin)
+from spyderlib.py3compat import PY3
+
 
 #-----------------------------------------------------------------------------
 # Templates
@@ -72,19 +80,20 @@ class IPythonControlWidget(TracebackLinksMixin, InspectObjectMixin, QTextEdit,
         BaseEditMixin.__init__(self)
         TracebackLinksMixin.__init__(self)
         InspectObjectMixin.__init__(self)
-        self.found_results = []
+
         self.calltip_widget = CallTipWidget(self, hide_timer_on=True)
+        self.found_results = []
+
         # To not use Spyder calltips obtained through the monitor
         self.calltips = False
-    
+
     def showEvent(self, event):
         """Reimplement Qt Method"""
         self.emit(SIGNAL("visibility_changed(bool)"), True)
-    
+
     def _key_question(self, text):
         """ Action for '?' and '(' """
-        parent = self.parentWidget()
-        self.current_prompt_pos = parent._prompt_pos
+        self.current_prompt_pos = self.parentWidget()._prompt_pos
         if self.get_current_line_to_cursor():
             last_obj = self.get_last_obj()
             if last_obj and not last_obj.isdigit():
@@ -94,7 +103,6 @@ class IPythonControlWidget(TracebackLinksMixin, InspectObjectMixin, QTextEdit,
     def keyPressEvent(self, event):
         """Reimplement Qt Method - Basic keypress event handler"""
         event, text, key, ctrl, shift = restore_keyevent(event)
-        
         if key == Qt.Key_Question and not self.has_selected_text():
             self._key_question(text)
         elif key == Qt.Key_ParenLeft and not self.has_selected_text():
@@ -183,7 +191,7 @@ class IPythonShellWidget(RichIPythonWidget):
         self.ipyclient = ipyclient
         self.exit_requested.connect(ipyclient.exit_callback)
     
-    def show_banner(self):
+    def long_banner(self):
         """Banner for IPython widgets with pylab message"""
         from IPython.core.usage import default_gui_banner
         banner = default_gui_banner
@@ -192,19 +200,9 @@ class IPythonShellWidget(RichIPythonWidget):
         autoload_pylab_o = CONF.get('ipython_console', 'pylab/autoload', True)
         mpl_installed = programs.is_module_installed('matplotlib')
         if mpl_installed and (pylab_o and autoload_pylab_o):
-            backend_o = CONF.get('ipython_console', 'pylab/backend', 0)
-            backends = {0: 'module://IPython.zmq.pylab.backend_inline',
-                        1: 'Qt4Agg', 2: 'Qt4Agg', 3: 'MacOSX', 4: 'GTKAgg',
-                        5: 'WXAgg', 6: 'TKAgg'}
-            pylab_013_message = """
-Welcome to pylab, a matplotlib-based Python environment [backend: %s].
-For more information, type 'help(pylab)'.\n""" % backends[backend_o]
-            pylab_1_message = """
-Populating the interactive namespace from numpy and matplotlib"""
-            if programs.is_module_installed('IPython', '>=1.0'):
-                banner = banner + pylab_1_message
-            else:
-                banner = banner + pylab_013_message
+            pylab_message = ("\nPopulating the interactive namespace from "
+                             "numpy and matplotlib")
+            banner = banner + pylab_message
         
         sympy_o = CONF.get('ipython_console', 'symbolic_math', True)
         if sympy_o:
@@ -219,19 +217,24 @@ These commands were executed:
             banner = banner + lines
         return banner
     
+    def short_banner(self):
+        """Short banner with Python and IPython versions"""
+        from IPython.core.release import version
+        py_ver = '%d.%d.%d' % (sys.version_info[0], sys.version_info[1],
+                               sys.version_info[2])
+        banner = 'Python %s on %s -- IPython %s' % (py_ver, sys.platform,
+                                                    version)
+        return banner
+    
     def clear_console(self):
         self.execute("%clear")
         
     def write_to_stdin(self, line):
-        """
-        Send raw characters to the IPython kernel through stdin
-        but only if the kernel is currently looking for raw input.
-        """
-        if self._reading:
-            if programs.is_module_installed('IPython', '>=1.0'):
-                self.kernel_client.stdin_channel.input(line)
-            else:
-                self.kernel_manager.stdin_channel.input(line)
+        """Send raw characters to the IPython kernel through stdin"""
+        try:
+            self.kernel_client.stdin_channel.input(line)
+        except AttributeError:
+            self.kernel_client.input(line)
     
     def set_background_color(self):
         lightbg_o = CONF.get('ipython_console', 'light_color')
@@ -244,8 +247,51 @@ These commands were executed:
                                   parent=self)
         clear_console = create_shortcut(self.clear_console, context='Console',
                                         name='Clear shell', parent=self)
+
+        # Fixed shortcuts
+        new_shortcut("Ctrl+T", self,
+                     lambda: self.emit(SIGNAL("new_ipyclient()")))
+
         return [inspect, clear_console]
 
+    def clean_invalid_var_chars(self, var):
+        """
+        Replace invalid variable chars in a string by underscores
+
+        Taken from http://stackoverflow.com/a/3305731/438386
+        """
+        if PY3:
+            return re.sub('\W|^(?=\d)', '_', var, re.UNICODE)
+        else:
+            return re.sub('\W|^(?=\d)', '_', var)
+
+    def get_signature(self, content):
+        """Get signature from inspect reply content"""
+        data = content.get('data', {})
+        text = data.get('text/plain', '')
+        if text:
+            text = ANSI_OR_SPECIAL_PATTERN.sub('', text)
+            self._control.current_prompt_pos = self._prompt_pos
+            line = self._control.get_current_line_to_cursor()
+            name = line[:-1].split('(')[-1]   # Take last token after a (
+            name = name.split('.')[-1]        # Then take last token after a .
+            # Clean name from invalid chars
+            try:
+                name = self.clean_invalid_var_chars(name).split('_')[-1]
+            except:
+                pass
+            argspec = getargspecfromtext(text)
+            if argspec:
+                # This covers cases like np.abs, whose docstring is
+                # the same as np.absolute and because of that a proper
+                # signature can't be obtained correctly
+                signature = name + argspec
+            else:
+                signature = getsignaturefromtext(text, name)
+            return signature
+        else:
+            return ''
+
     #---- IPython private methods ---------------------------------------------
     def _context_menu_make(self, pos):
         """Reimplement the IPython context menu"""
@@ -259,30 +305,23 @@ These commands were executed:
         """
         banner_o = CONF.get('ipython_console', 'show_banner', True)
         if banner_o:
-            return self.show_banner()
+            return self.long_banner()
         else:
-            return ''
+            return self.short_banner()
     
     def _handle_object_info_reply(self, rep):
         """
         Reimplement call tips to only show signatures, using the same style
-        from our Editor and External Console too.
+        from our Editor and External Console too
+        Note: For IPython 2-
         """
         self.log.debug("oinfo: %s", rep.get('content', ''))
         cursor = self._get_cursor()
         info = self._request_info.get('call_tip')
         if info and info.id == rep['parent_header']['msg_id'] and \
-                info.pos == cursor.position():
-            # Get the information for a call tip.  For now we format the call
-            # line as string, later we can pass False to format_call and
-            # syntax-highlight it ourselves for nicer formatting in the
-            # calltip.
+          info.pos == cursor.position():
             content = rep['content']
-            # if this is from pykernel, 'docstring' will be the only key
             if content.get('ismagic', False):
-                # Don't generate a call-tip for magics. Ideally, we should
-                # generate a tooltip, but not on ( like we do for actual
-                # callables.
                 call_info, doc = None, None
             else:
                 call_info, doc = call_tip(content, format_call=True)
@@ -299,6 +338,23 @@ These commands were executed:
             if call_info:
                 self._control.show_calltip(_("Arguments"), call_info,
                                            signature=True, color='#2D62FF')
+
+    def _handle_inspect_reply(self, rep):
+        """
+        Reimplement call tips to only show signatures, using the same style
+        from our Editor and External Console too
+        Note: For IPython 3+
+        """
+        cursor = self._get_cursor()
+        info = self._request_info.get('call_tip')
+        if info and info.id == rep['parent_header']['msg_id'] and \
+          info.pos == cursor.position():
+            content = rep['content']
+            if content.get('status') == 'ok' and content.get('found', False):
+                signature = self.get_signature(content)
+                if signature:
+                    self._control.show_calltip(_("Arguments"), signature,
+                                               signature=True, color='#2D62FF')
     
     #---- Qt methods ----------------------------------------------------------
     def focusInEvent(self, event):
@@ -326,16 +382,25 @@ class IPythonClient(QWidget, SaveHistoryMixin):
     
     SEPARATOR = '%s##---(%s)---' % (os.linesep*2, time.ctime())
     
-    def __init__(self, plugin, history_filename, connection_file=None,
+    def __init__(self, plugin, name, history_filename, connection_file=None, 
+                 hostname=None, sshkey=None, password=None, 
                  kernel_widget_id=None, menu_actions=None):
         super(IPythonClient, self).__init__(plugin)
         SaveHistoryMixin.__init__(self)
         self.options_button = None
-
+        
+        # stop button and icon
+        self.stop_button = None
+        self.stop_icon = get_icon("stop.png")
+        
         self.connection_file = connection_file
         self.kernel_widget_id = kernel_widget_id
-        self.name = ''
-        self.shellwidget = IPythonShellWidget(config=plugin.ipywidget_config(),
+        self.hostname = hostname
+        self.sshkey = sshkey
+        self.password = password
+        self.name = name
+        self.get_option = plugin.get_option
+        self.shellwidget = IPythonShellWidget(config=self.shellwidget_config(),
                                               local_kernel=False)
         self.shellwidget.hide()
         self.infowidget = WebView(self)
@@ -359,7 +424,7 @@ class IPythonClient(QWidget, SaveHistoryMixin):
         vlayout.addWidget(self.infowidget)
         self.setLayout(vlayout)
         
-        self.exit_callback = lambda: plugin.close_console(client=self)
+        self.exit_callback = lambda: plugin.close_client(client=self)
         
     #------ Public API --------------------------------------------------------
     def show_shellwidget(self, give_focus=True):
@@ -374,15 +439,37 @@ class IPythonClient(QWidget, SaveHistoryMixin):
         self.shellwidget.set_ipyclient(self)
         
         # To save history
-        self.shellwidget.executing.connect(
-                                      lambda c: self.add_to_history(command=c))
+        self.shellwidget.executing.connect(self.add_to_history)
+        
+        # For Mayavi to run correctly
+        self.shellwidget.executing.connect(self.set_backend_for_mayavi)
         
         # To update history after execution
         self.shellwidget.executed.connect(self.update_history)
         
         # To update the Variable Explorer after execution
         self.shellwidget.executed.connect(self.auto_refresh_namespacebrowser)
+        
+        # To show a stop button, when executing a process
+        self.shellwidget.executing.connect(self.enable_stop_button)
+        
+        # To hide a stop button after execution stopped
+        self.shellwidget.executed.connect(self.disable_stop_button)
     
+    def enable_stop_button(self):
+        self.stop_button.setEnabled(True)
+   
+    def disable_stop_button(self):
+        self.stop_button.setDisabled(True)
+        
+    def stop_button_click_handler(self):
+        self.stop_button.setDisabled(True)
+        # Interrupt computations or stop debugging
+        if not self.shellwidget._reading:
+            self.interrupt_kernel()
+        else:
+            self.shellwidget.write_to_stdin('exit')
+
     def show_kernel_error(self, error):
         """Show kernel initialization errors in infowidget"""
         # Remove explanation about how to kill the kernel (doesn't apply to us)
@@ -416,7 +503,8 @@ class IPythonClient(QWidget, SaveHistoryMixin):
     
     def get_name(self):
         """Return client name"""
-        return _("Console") + " " + self.name
+        return ((_("Console") if self.hostname is None else self.hostname)
+                + " " + self.name)
     
     def get_control(self):
         """Return the text widget (or similar) to give focus to"""
@@ -429,19 +517,17 @@ class IPythonClient(QWidget, SaveHistoryMixin):
 
     def get_options_menu(self):
         """Return options menu"""
-        # Kernel
-        self.interrupt_action = create_action(self, _("Interrupt kernel"),
-                                              icon=get_icon('terminate.png'),
-                                              triggered=self.interrupt_kernel)
-        self.restart_action = create_action(self, _("Restart kernel"),
-                                            icon=get_icon('restart.png'),
-                                            triggered=self.restart_kernel)
+        restart_action = create_action(self, _("Restart kernel"),
+                                       shortcut=QKeySequence("Ctrl+."),
+                                       icon=get_icon('restart.png'),
+                                       triggered=self.restart_kernel)
+        restart_action.setShortcutContext(Qt.WidgetWithChildrenShortcut)
+        
         # Main menu
         if self.menu_actions is not None:
-            actions = [self.interrupt_action, self.restart_action, None] +\
-                      self.menu_actions
+            actions = [restart_action, None] + self.menu_actions
         else:
-            actions = [self.interrupt_action, self.restart_action]
+            actions = [restart_action]
         return actions
     
     def get_toolbar_buttons(self):
@@ -449,6 +535,17 @@ class IPythonClient(QWidget, SaveHistoryMixin):
         #TODO: Eventually add some buttons (Empty for now)
         # (see for example: spyderlib/widgets/externalshell/baseshell.py)
         buttons = []
+        # Code to add the stop button 
+        if self.stop_button is None:
+            self.stop_button = create_toolbutton(self, text=_("Stop"),
+                                             icon=self.stop_icon,
+                                             tip=_("Stop the current command"))
+            self.disable_stop_button()
+            # set click event handler
+            self.stop_button.clicked.connect(self.stop_button_click_handler)
+        if self.stop_button is not None:
+            buttons.append(self.stop_button)
+            
         if self.options_button is None:
             options = self.get_options_menu()
             if options:
@@ -460,8 +557,9 @@ class IPythonClient(QWidget, SaveHistoryMixin):
                 self.options_button.setMenu(menu)
         if self.options_button is not None:
             buttons.append(self.options_button)
+
         return buttons
-    
+
     def add_actions_to_context_menu(self, menu):
         """Add actions to IPython widget context menu"""
         # See spyderlib/widgets/ipython.py for more details on this method
@@ -526,6 +624,19 @@ class IPythonClient(QWidget, SaveHistoryMixin):
     def update_history(self):
         self.history = self.shellwidget._history
     
+    def set_backend_for_mayavi(self, command):
+        calling_mayavi = False
+        lines = command.splitlines()
+        for l in lines:
+            if not l.startswith('#'):
+                if 'import mayavi' in l or 'from mayavi' in l:
+                    calling_mayavi = True
+                    break
+        if calling_mayavi:
+            message = _("Changing backend to Qt for Mayavi")
+            self.shellwidget._append_plain_text(message + '\n')
+            self.shellwidget.execute("%gui inline\n%gui qt")
+    
     def interrupt_message(self):
         """
         Print an interrupt message when the client is connected to an external
@@ -553,6 +664,64 @@ class IPythonClient(QWidget, SaveHistoryMixin):
         if self.namespacebrowser:
             self.namespacebrowser.refresh_table()
     
+    def shellwidget_config(self):
+        """Generate a Config instance for shell widgets using our config
+        system
+        
+        This lets us create each widget with its own config (as opposed to
+        IPythonQtConsoleApp, where all widgets have the same config)
+        """
+        # ---- IPython config ----
+        try:
+            profile_path = osp.join(get_ipython_dir(), 'profile_default')
+            full_ip_cfg = load_pyconfig_files(['ipython_qtconsole_config.py'],
+                                              profile_path)
+            
+            # From the full config we only select the IPythonWidget section
+            # because the others have no effect here.
+            ip_cfg = Config({'IPythonWidget': full_ip_cfg.IPythonWidget})
+        except:
+            ip_cfg = Config()
+       
+        # ---- Spyder config ----
+        spy_cfg = Config()
+        
+        # Make the pager widget a rich one (i.e a QTextEdit)
+        spy_cfg.IPythonWidget.kind = 'rich'
+        
+        # Gui completion widget
+        gui_comp_o = self.get_option('use_gui_completion')
+        completions = {True: 'droplist', False: 'ncurses'}
+        spy_cfg.IPythonWidget.gui_completion = completions[gui_comp_o]
+
+        # Pager
+        pager_o = self.get_option('use_pager')
+        if pager_o:
+            spy_cfg.IPythonWidget.paging = 'inside'
+        else:
+            spy_cfg.IPythonWidget.paging = 'none'
+        
+        # Calltips
+        calltips_o = self.get_option('show_calltips')
+        spy_cfg.IPythonWidget.enable_calltips = calltips_o
+
+        # Buffer size
+        buffer_size_o = self.get_option('buffer_size')
+        spy_cfg.IPythonWidget.buffer_size = buffer_size_o
+        
+        # Prompts
+        in_prompt_o = self.get_option('in_prompt')
+        out_prompt_o = self.get_option('out_prompt')
+        if in_prompt_o:
+            spy_cfg.IPythonWidget.in_prompt = in_prompt_o
+        if out_prompt_o:
+            spy_cfg.IPythonWidget.out_prompt = out_prompt_o
+        
+        # Merge IPython and Spyder configs. Spyder prefs will have prevalence
+        # over IPython ones
+        ip_cfg._merge(spy_cfg)
+        return ip_cfg
+    
     #------ Private API -------------------------------------------------------
     def _create_loading_page(self):
         loading_template = Template(LOADING)
@@ -571,9 +740,6 @@ class IPythonClient(QWidget, SaveHistoryMixin):
         Reimplement Qt method to stop sending the custom_restart_kernel_died
         signal
         """
-        if programs.is_module_installed('IPython', '>=1.0'):
-            kc = self.shellwidget.kernel_client
-            if kc is not None:
-                kc.hb_channel.pause()
-        else:
-            self.shellwidget.custom_restart = False
+        kc = self.shellwidget.kernel_client
+        if kc is not None:
+            kc.hb_channel.pause()
